import {
    Button,
    Card,
    CardContent,
    Divider,
    Grid,
    Paper,
    Stack,
    Typography,
} from '@mui/material'
import { Fragment, useState } from 'react'
import AddIcon from '@mui/icons-material/Add'
import { MapContainer, Marker, Polyline, Popup, TileLayer } from 'react-leaflet'
import mapConfig from '../../config/mapConfig'
import TextPath from 'react-leaflet-textpath'
import PlaintextUpload from './PlaintextUpload'
import FileUpload from './FileUpload'
import L from 'leaflet'
import DeleteIcon from '@mui/icons-material/Delete'
import { PathDto } from '../../swagger/data-contracts'
import { formatHtmlStringToReactDom } from '../../utils/formatting/HtmlUtils'

// Page with tracking tool
const TrackingTool = () => {
    // Path response from the API
    const [pathDto, setPathDto] = useState<PathDto | undefined>(undefined)

    const createDummyPathCoords = () => {
        // Sample dummy path to display
        const dummyPath = []
        for (let i = 0; i < 10; i += 1) {
            dummyPath.push({
                latitude:
                    mapConfig.defaultCoordinates[0] +
                    i * 0.1 +
                    Math.random() * 0.1,
                longitude:
                    mapConfig.defaultCoordinates[1] +
                    i * 0.1 +
                    Math.random() * 0.1,
            })
        }
        return dummyPath
    }

    const createDummyPath = () => {
        const coords = createDummyPathCoords()
        const polylines: any[] = []

        if (coords.length < 2) {
            return []
        }

        for (let i = 0; i < coords.length - 1; i += 1) {
            polylines.push(
                <TextPath
                    positions={[
                        [coords[i].latitude, coords[i].longitude],
                        [coords[i + 1].latitude, coords[i + 1].longitude],
                    ]}
                    text="►"
                    attributes={{
                        'font-size': 25,
                        fill: 'blue',
                    }}
                    repeat
                    center
                    weight={10}
                ></TextPath>
            )
        }

        return [polylines, coords]
    }

    const [polylines, coords] = createDummyPath()

    return (
        <Fragment>
            <Typography variant="h3" sx={{ mb: 2 }} fontWeight="bold">
                Tracking Tool
            </Typography>

            {pathDto && (
                <Fragment>
                    <Card variant="outlined">
                        <CardContent>
                            <Stack direction="column">
                                <Typography
                                    variant="h5"
                                    sx={{ mb: 1 }}
                                    fontWeight="600"
                                >
                                    Text
                                </Typography>
                                <Typography variant="body2">
                                    {formatHtmlStringToReactDom(
                                        pathDto.text ?? ''
                                    )}
                                </Typography>
                            </Stack>
                        </CardContent>
                    </Card>
                </Fragment>
            )}
            <Grid container>
                <Grid item xs={12}>
                    {pathDto && pathDto?.foundCatalogItems?.length === 0 && (
                        <Typography
                            variant="body1"
                            sx={{ mb: 2 }}
                            fontWeight="500"
                            align="center"
                            color="error.main"
                        >
                            Looks like no path / catalog items match this query.
                        </Typography>
                    )}

                    <Stack
                        direction="row"
                        alignItems="flex-start"
                        spacing={2}
                        sx={{ mt: 1 }}
                    >
                        <Typography
                            variant="h5"
                            sx={{ mb: 2 }}
                            fontWeight="500"
                        >
                            {pathDto ? 'Update path' : 'Show Path'}
                        </Typography>
                        <PlaintextUpload setPaths={setPathDto} />
                        <FileUpload />
                    </Stack>
                </Grid>

                <Grid
                    item
                    xs={12}
                    md={12}
                    style={{
                        minHeight: '60vh',
                        maxHeight: '100vh',
                        width: '100%',
                    }}
                >
                    <MapContainer
                        center={[
                            mapConfig.defaultCoordinates[0],
                            mapConfig.defaultCoordinates[1],
                        ]}
                        zoom={mapConfig.defaultZoom}
                        style={{ height: '100%', minHeight: '100%' }}
                    >
                        <TileLayer
                            attribution={mapConfig.attribution}
                            url={mapConfig.url}
                        />
                        {coords.map(({ latitude, longitude }, idx) => (
                            <Marker position={[latitude, longitude]} />
                        ))}
                        {polylines}
                    </MapContainer>
                </Grid>
            </Grid>
        </Fragment>
    )
}

export default TrackingTool
